program test_set_max_speed

  ! This program reads cont_list and cont_list_proj created by
  ! get_1_outerm, and it aloso reads ssh, u, v. We assume that the
  ! coordinates x, y in cont_list_proj are relative to the grid of
  ! ssh, u, v.

  ! Libraries:
  use contour_531, only: null_polyline, polyline
  use gpc_f, only: shp_read_pol, polygon
  use jumble, only: get_command_arg_dyn, assert, deg_to_rad
  use shapelib, only: shpfileobject, shpclose
  use shapelib_03, only: shp_open_03, shp_get_info_03, dbf_read_attribute_03

  use config_m, only: config
  use cont_list_m, only: create_cont_list, close_cont_list
  use derived_types, only: eddy, shpc_slice_handler, ssh_contour
  use input_ssh_m, only: input_ssh, corner, nlon, nlat
  use read_eddy_m, only: read_eddy
  use set_max_speed_m, only: set_max_speed
  use shpc_close_m, only: shpc_close
  use shpc_create_m, only: shpc_create
  use shpc_open_m, only: shpc_open
  use write_eddy_m, only: write_eddy

  implicit none

  logical cyclone
  logical periodic ! grid is periodic in longitude
  integer d, eddy_index, n_cont, i, ishape
  real, allocatable:: ssh(:, :) ! (nlon, nlat) sea-surface height, in m
  real, allocatable:: u(:, :), v(:, :) ! (nlon, nlat) wind, in m s-1
  type(eddy) e
  real step(2) ! in rad
  TYPE(shpc_slice_handler) hshpc
  character(len = :), allocatable:: path

  ! Window around the extremum:
  integer llc(2) ! indices in global grid of lower left corner
  integer urc(2) ! indices in global grid of upper right corner

  integer, parameter:: n_max_cont = 31

  type(polyline) cont_list(n_max_cont)
  ! Contour list, with points given by longitude and latitude, in
  ! radians. Defined only for subscripts 1:n_cont. The order of
  ! points in each contour (clockwise or counter-clockwise) is not
  ! specified.

  type(polyline) cont_list_proj(n_max_cont)
  ! Contour list, with points given by projection coordinates. Defined
  ! only for subscripts 1:n_cont. The contour with a given subscript
  ! in cont_list_proj corresponds to the contour with the same
  ! subscript in cont_list. More precisely, the point with subscript j
  ! in contour i of cont_list_proj corresponds to the point with
  ! subscript j in contour i of cont_list (this is important for
  ! mean_speed). For i <= n_cont, the order of points in
  ! cont_list_proj(i) (clockwise or counter-clockwise) is not
  ! specified.

  real cont_list_ssh(n_max_cont)
  TYPE(shpfileobject) hshp
  type(polygon) p
  character(len = :), allocatable:: ssh_fname, uv_fname ! file names

  !----------------------------------------------------------------

  print *, "test_set_max_speed: cyclone = ? "
  read *, cyclone
  call config
  call get_command_arg_dyn(1, ssh_fname, "Required arguments: h-file uv-file")
  call get_command_arg_dyn(2, uv_fname, "Required arguments: h-file uv-file")
  call input_ssh(step, periodic, ssh, u, v, "SHPC", ssh_fname, uv_fname, &
       uv_fname)
  print *, "test_set_max_speed: Reading from shapefiles..."
  call get_command_arg_dyn(3, path, "Required 3rd argument: SHPC-directory")
  call shpc_open(hshpc, path, cyclone, slice = 0, pszaccess = "rb")
  call read_eddy(e, d, eddy_index, hshpc, ishape = 0)
  CALL shpc_close(hshpc)

  if (e%out_cont%closed) then
     e%extr%coord_proj = nint((e%extr%coord - corner) / step) + 1

     call get_command_arg_dyn(4, path, &
          "Required 4th argument: cont-list-shapefile")
     call shp_open_03(hshp, path, "rb")
     call shp_get_info_03(hshp, n_cont)
     call assert(1 <= n_cont, n_cont <= n_max_cont - 1, &
          "test_set_max_speed n_cont")

     do i = 1, n_cont
        ishape = i - 1
        call shp_read_pol(p, hshp, ishape)

        if (p%nparts == 0) then
           cont_list(i) = null_polyline()
        else
           cont_list(i) = p%part(1)
           cont_list(i)%points = cont_list(i)%points * deg_to_rad
        end if

     end do

     call shpclose(hshp)
     call get_command_arg_dyn(5, path, &
          "Required 5th argument: cont-list-proj-shapefile")
     call shp_open_03(hshp, path, "rb")

     do i = 1, n_cont
        ishape = i - 1
        call shp_read_pol(p, hshp, ishape)

        if (p%nparts == 0) then
           cont_list_proj(i) = null_polyline()
        else
           cont_list_proj(i) = p%part(1)
        end if

        call dbf_read_attribute_03(cont_list_ssh(i), hshp, ifield = 0, &
             ishape = ishape)
     end do

     call shpclose(hshp)

     ! Restrict the field to the outermost contour:
     llc = floor(minval(cont_list_proj(n_cont)%points, dim = 2))
     urc = ceiling(maxval(cont_list_proj(n_cont)%points, dim = 2))

     ! Should have no effect except because of roundup error:
     urc(2) = min(urc(2), nlat)
     if (.not. periodic) urc(1) = min(urc(1), nlon)

     call create_cont_list
     call set_max_speed(e%speed_cont, e%max_speed, cont_list, cont_list_proj, &
          cont_list_ssh, n_cont, e%extr%coord, real(e%extr%coord_proj), &
          ssh(llc(1):urc(1), llc(2):urc(2)), u(llc(1):urc(1), llc(2):urc(2)), &
          v(llc(1):urc(1), llc(2):urc(2)), real(llc), step)
     call close_cont_list
     call shpc_create(hshpc, shpc_dir = "SHPC", cyclone = cyclone, slice = 0, &
          grid_lon_lat = .true.)
     call write_eddy(e, hshpc, d, eddy_index)
     write(hshpc%unit, fmt = *) 0
     CALL shpc_close(hshpc)
     print *, 'test_set_max_speed: Created SHPC and shapefiles cont_list ', &
          'and cont_list_proj.'
  else
     print *, "No outermost contour found."
  end if

end program test_set_max_speed
